@@ -11,12 +11,23 @@ use Type::Utils   -all;
 
 use Data::Object;
 
-our $VERSION = '0.03'; # VERSION
+our $VERSION = '0.04'; # VERSION
 
 extends 'Types::Standard';
 extends 'Types::Common::Numeric';
 extends 'Types::Common::String';
 
+my $registry = __PACKAGE__->meta;
+
+my $constraint_name;
+my $constraint_code;
+
+my $coercion_name;
+my $coercion_code;
+
+my $explanation_name;
+my $explanation_code;
+
 sub DECLARE {
     my ($name, %opts) = @_;
 
@@ -24,8 +35,6 @@ sub DECLARE {
 
     ($opts{name} = $name) =~ s/:://g;
 
-    my $registry = __PACKAGE__->meta;
-
     my @cans = ref($opts{can})  eq 'ARRAY' ? @{$opts{can}}  : $opts{can}  // ();
     my @isas = ref($opts{isa})  eq 'ARRAY' ? @{$opts{isa}}  : $opts{isa}  // ();
     my @does = ref($opts{does}) eq 'ARRAY' ? @{$opts{does}} : $opts{does} // ();
@@ -94,42 +103,79 @@ sub DECLARE {
     return $type;
 }
 
-DECLARE ["ArrayObj", "ArrayObject"] => (
+$constraint_name = 'constraint_generator';
+$constraint_code = sub {
+    my $param = @_ ? Types::TypeTiny::to_TypeTiny(shift) :
+        return $registry->get_type('ArrayObject');
+
+    Types::TypeTiny::TypeTiny->check($param)
+        or Types::Standard::_croak(
+            "Parameter to ArrayObject[`a] expected ".
+            "to be a type constraint; got $param"
+        );
+
+    return sub {
+        my $arrayobj = shift;
+        $param->check($_) || return for @$arrayobj;
+        return !!1;
+    }
+};
+
+$coercion_name = 'coercion_generator';
+$coercion_code = sub {
+    my ($parent, $child, $param) = @_;
+
+    return $parent->coercion unless $param->has_coercion;
+
+    my $coercable_item = $param->coercion->_source_type_union;
+    my $c = "Type::Coercion"->new(type_constraint => $child);
+
+    $c->add_type_coercions(
+        $registry->get_type('ArrayRef') => sub {
+            my $value = @_ ? $_[0] : $_;
+            my $new   = [];
+
+            for (my $i=0; $i < @$value; $i++) {
+                my $item = $value->[$i];
+                return $value unless $coercable_item->check($item);
+                $new->[$i] = $param->coerce($item);
+            }
+
+            return $parent->coerce($new);
+        },
+    );
+
+    return $c;
+};
+
+$explanation_name = 'deep_explanation';
+$explanation_code = sub {
+    my ($type, $value, $varname) = @_;
+    my $param = $type->parameters->[0];
+
+    for my $i (0 .. $#$value) {
+        my $item = $value->[$i];
+        next if $param->check($item);
+        my $message  = '"%s" constrains each value in the array object with "%s"';
+        my $position = sprintf('%s->[%d]', $varname, $i);
+        my $criteria = $param->validate_explain($item, $position);
+        return [sprintf($message, $type, $param), @{$criteria}]
+    }
+
+    return;
+};
+
+my @with_array_extras = (
+    $constraint_name  => $constraint_code,
+    $coercion_name    => $coercion_code,
+    $explanation_name => $explanation_code,
+);
+
+DECLARE ["ArrayObj", "ArrayObject"] => (@with_array_extras,
     isa    => ["Data::Object::Array"],
     does   => ["Data::Object::Role::Array"],
     can    => ["data", "dump"],
     coerce => ["ArrayRef"],
-    constraint_generator => sub {
-        return Data::Object::Library::ArrayObject() unless @_;
-
-        my $param = Types::TypeTiny::to_TypeTiny(shift);
-
-        Types::TypeTiny::TypeTiny->check($param)
-            or Types::Standard::_croak(
-                "Parameter to ArrayObject[`a] expected ".
-                "to be a type constraint; got $param"
-            );
-
-        return sub {
-            my $arrayobj = shift;
-            $param->check($_) || return for @$arrayobj;
-            return !!1;
-        }
-    },
-    deep_explanation => sub {
-        my ($type, $value, $varname) = @_;
-        my $param = $type->parameters->[0];
-
-        for my $i (0 .. $#$value) {
-            my $item = $value->[$i];
-            next if $param->check($item);
-            my $message  = '"%s" constrains each value in the array with "%s"';
-            my @criteria = @{ $param->validate_explain($item, sprintf('%s->[%d]', $varname, $i)) };
-            return [sprintf($message, $type, $param), @criteria]
-        }
-
-        return;
-    },
 );
 
 DECLARE ["CodeObj", "CodeObject"] => (
@@ -146,63 +192,101 @@ DECLARE ["FloatObj", "FloatObject"] => (
     coerce => ["Str", "Num", "LaxNum"],
 );
 
-DECLARE ["HashObj", "HashObject"] => (
+$constraint_name = 'constraint_generator';
+$constraint_code = sub {
+    my $param = @_ ? Types::TypeTiny::to_TypeTiny(shift) :
+        return $registry->get_type('HashObject');
+
+    Types::TypeTiny::TypeTiny->check($param)
+        or Types::Standard::_croak(
+            "Parameter to HashObject[`a] expected ".
+            "to be a type constraint; got $param"
+        );
+
+    return sub {
+        my $hashobj = shift;
+        $param->check($_) || return for values %$hashobj;
+        return !!1;
+    }
+};
+
+$coercion_name = 'coercion_generator';
+$coercion_code = sub {
+    my ($parent, $child, $param) = @_;
+
+    return $parent->coercion unless $param->has_coercion;
+
+    my $coercable_item = $param->coercion->_source_type_union;
+    my $c = "Type::Coercion"->new(type_constraint => $child);
+
+    $c->add_type_coercions(
+        $registry->get_type('HashRef') => sub {
+            my $value = @_ ? $_[0] : $_;
+            my $new   = {};
+
+            for my $key (sort keys %$value) {
+                my $item = $value->{$key};
+                return $value unless $coercable_item->check($item);
+                $new->{$key} = $param->coerce($item);
+            }
+
+            return $parent->coerce($new);
+        },
+    );
+
+    return $c;
+};
+
+$explanation_name = 'deep_explanation';
+$explanation_code = sub {
+    my ($type, $value, $varname) = @_;
+    my $param = $type->parameters->[0];
+
+    for my $key (sort keys %$value) {
+        my $item = $value->{$key};
+        next if $param->check($item);
+        my $message  = '"%s" constrains each value in the hash object with "%s"';
+        my $position = sprintf('%s->{%s}', $varname, B::perlstring($key));
+        my $criteria = $param->validate_explain($item, $position);
+        return [sprintf($message, $type, $param), @{$criteria}]
+    }
+
+    return;
+};
+
+my $overrides_name = 'my_methods';
+my $overrides_opts = {
+    hashref_allows_key => sub {
+        my ($self, $key) = @_;
+        $registry->get_type('Str')->check($key);
+    },
+    hashref_allows_value => sub {
+        my ($self, $key, $value) = @_;
+
+        return !!0 unless $self->my_hashref_allows_key($key);
+        return !!1 if $self == $registry->get_type('HashRef');
+
+        my $href = $self->find_parent(sub {
+            $_->has_parent && $_->parent == $registry->get_type('HashRef')
+        });
+
+        my $param = $href->type_parameter;
+        $registry->get_type('Str')->check($key) and $param->check($value);
+    },
+};
+
+my @with_hash_extras = (
+    $constraint_name  => $constraint_code,
+    $coercion_name    => $coercion_code,
+    $explanation_name => $explanation_code,
+    $overrides_name   => $overrides_opts,
+);
+
+DECLARE ["HashObj", "HashObject"] => (@with_hash_extras,
     isa    => ["Data::Object::Hash"],
     does   => ["Data::Object::Role::Hash"],
     can    => ["data", "dump"],
     coerce => ["HashRef"],
-    constraint_generator => sub {
-        return Data::Object::Library::HashObject() unless @_;
-
-        my $param = Types::TypeTiny::to_TypeTiny(shift);
-
-        Types::TypeTiny::TypeTiny->check($param)
-            or Types::Standard::_croak(
-                "Parameter to HashObject[`a] expected ".
-                "to be a type constraint; got $param"
-            );
-
-        return sub {
-            my $hashobj = shift;
-            $param->check($_) || return for values %$hashobj;
-            return !!1;
-        }
-    },
-    deep_explanation => sub {
-        my ($type, $value, $varname) = @_;
-        my $param = $type->parameters->[0];
-
-        for my $k (sort keys %$value) {
-            my $item = $value->{$k};
-            next if $param->check($item);
-            my $message  = '"%s" constrains each value in the hash object with "%s"';
-            my @criteria = @{ $param->validate_explain($item, sprintf('%s->{%s}', $varname, B::perlstring($k))) };
-            return [sprintf($message, $type, $param), @criteria]
-        }
-
-        return;
-    },
-    my_methods => {
-        hashref_allows_key => sub {
-            my $self = shift;
-            Data::Object::Library::Str()->check($_[0]);
-        },
-        hashref_allows_value => sub {
-            my $self = shift;
-            my ($key, $value) = @_;
-
-            return !!0 unless $self->my_hashref_allows_key($key);
-            return !!1 if $self == Data::Object::Library::HashRef();
-
-            my $href  = $self->find_parent(sub {
-                $_->has_parent && $_->parent == Data::Object::Library::HashRef()
-            });
-
-            my $param = $href->type_parameter;
-
-            Data::Object::Library::Str()->check($key) and $param->check($value);
-        },
-    },
 );
 
 DECLARE ["IntObj", "IntObject", "IntegerObj", "IntegerObject"] => (
@@ -268,7 +352,7 @@ Data::Object::Library - Data::Object Type Library for Perl 5
 
 =head1 VERSION
 
-version 0.03
+version 0.04
 
 =head1 SYNOPSIS
 
